/*
 * Copyright (C) 2000-2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 */

package estella.lib;
import  estella.lib.Log;
import  estella.lib.Constantes;

import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Random;


/**
* Gerencia descritor
* @author Daniel Machado
* @see 02/12/2005
*/
public class Descritor
{
	private static short[] clientID = null;
	private static long bootTime;
	private static long counter = 0x00000000;
	private Log LOG;
	private Constantes CON;
	private byte[] DESCRITOR;
		
	/**
	* Novo descritor
	* @author Daniel Machado
	*/
	public Descritor(byte[] id, byte tipo, int ttl, int hops, byte[] payload)
	{	
		int payload_length, i=0, j=0;
		LOG = new Log();
		CON = new Constantes();
		
		if(id == null)
			id = geraID();
		
		if(tipo != CON.PING) 
			payload_length = payload.length;
		else
			payload_length = 0x00000001;
		
		this.DESCRITOR = new byte[23 + payload_length];
		
		// Payload Descriptor ID (0~15)
		for(i=0; i<16; i++)
			this.DESCRITOR[i] = (byte)id[i];
		
		// Payload Descriptor (16)
		this.DESCRITOR[i] = (byte) tipo;
		
		// TTL (17)
		this.DESCRITOR[++i] = (byte) ttl;
		
		// Hops (18)
		this.DESCRITOR[++i] = (byte) hops;
		
		// Payload length (19~22)
		this.DESCRITOR[++i] = (byte)(payload_length);					// byte 19
		this.DESCRITOR[++i] = (byte)(payload_length >> 8);		// byte 20
		this.DESCRITOR[++i] = (byte)(payload_length >> 16);		// byte 21
		this.DESCRITOR[++i] = (byte)(payload_length >> 24);		// byte 22
		
		// Copia o payload para o descritor (23~payload_length)
		for(j=0, i=23; j<payload_length; i++, j++)
			this.DESCRITOR[i] = payload[j];
	}
	
	/**
	* Novo descritor
	* @author Daniel Machado
	*/
	public Descritor(byte tipo, int ttl, int hops, byte[] payload)
	{
		this(null, tipo, ttl, hops, payload);
	}
	
	public Descritor(String descritor)
	{
		this.DESCRITOR = (byte[]) descritor.getBytes();
	}
	
	public void id(byte[] _id)
	{
		for(int i=0; i<16; i++)
			DESCRITOR[i] = _id[i];
	}
	
	public String stringId()
	{
		String id = new String("");
		for (int i=0; i<16; i++)
    {
			int x = (0xFF & DESCRITOR[i]);
			id += x;
    }
		return id;
	}

	public byte[] id()
	{
		byte[] id = new byte[16];
		for(int i=0; i<16; i++)
			id[i] = DESCRITOR[i];
		return id;
	}
	
	/**
	* Generate something remotely resembling a windows guid
	* Essa funcao pertence originalmente ao software jTella versao 0.7 - cedida sobre GPL 
	* @author KenMcCrary
	*/
	public byte[] geraID()
	{
		Random rand = new Random();
		byte [] guid = { (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
                       (byte)(rand.nextInt(9)),
										};
    return guid;
	}
	
	/**
	* Retorna o descritor gerado
	* @return byte[] - retorna um array de bytes com o descritor, se ele nao foi criado, o retorno sera null
	*/
	public byte[] retorna()
	{
		return this.DESCRITOR;
	}
	
	/**
	* TTL - time to live
	* @return byte - retorna o ttl do descritor
	*/
	public int ttl()
	{
		return (int) (0xFF & DESCRITOR[17]);
	}
	
	/**
	* TTL -time to live
	* @param ttl - seta o ttl com o byte passado
	*/
	public void ttl(int ttl)
	{
		this.DESCRITOR[17] = (byte)ttl;
	}
	
	public byte[] payload()
	{
		int payload_length=0;
		int j=22;
		
		for(int i=19; i<22; i++)
			payload_length += DESCRITOR[i];
		
		byte[] payload = new byte[payload_length];
		
		for(int i=0; i<payload_length; i++)
			payload[i] = DESCRITOR[++j];
		return payload;
	}
		
	/**
	* HOPS - Seta o numero de vezes que o descritor foi roteado (passado por outros servents)
	* @param hops - byte com o hops a ser setado
	*/
	public void hops(int hops)
	{
		this.DESCRITOR[18] = (byte)hops;
	}
	
	/**
	* HOPS - Retorna o numero de vezes que o descritor foi roteado (passado por outros servents)
	* @return  hops - byte com o hops
	*/
	public int hops()
	{
		return (int) (0xFF & DESCRITOR[18]);
	}
	
	/**
	* Tipo do descritor
	* @return byte - tipo do descritor
	*/
	public int tipo()
	{
		return DESCRITOR[16];
	}
}
